unit PPCSync_Main;

{
  To do:
  - check if target dir exists ! otherwise ?
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, CETools, ExtCtrls;

type
  TForm1 = class(TForm)
    CEFindFile: TCEFindFile;
    Button1: TButton;
    edtDateTime: TLabeledEdit;
    edtDesktopMap: TLabeledEdit;
    edtPocketMap: TLabeledEdit;
    chbAllFiles: TCheckBox;
    CEFileOperations: TCEFileOperations;
    chbIncludeSubmaps: TCheckBox;
    procedure Button1Click(Sender: TObject);
    procedure edtDateTimeExit(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PathNameExit(Sender: TObject);
  private
    { Private declarations }
    function CopyFiles( sSourceMap, sTargetMap: string; tdLastCopied: TDateTime ): Boolean;
    function BatchCopyFiles( sSourceMap, sTargetMap: string; tdLastCopied: TDateTime; bIncludeSubmaps: Boolean = True ): Boolean;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses InifileLib;

{$R *.dfm}

function TForm1.CopyFiles( sSourceMap, sTargetMap: string; tdLastCopied: TDateTime ): Boolean;
var
  sr: TSearchRec;
  FileAttrs: Integer;
begin
  Result := True;
  FileAttrs := 0;
  FileAttrs := FileAttrs + faDirectory;
  FileAttrs := FileAttrs + faAnyFile;
  try
    if FindFirst( sSourceMap + '*.*', FileAttrs, sr ) = 0 then begin
      repeat
        if (sr.Attr and faDirectory ) = sr.Attr then begin
          Result := Result and CopyFiles( sSourceMap + sr.Name + '\', sTargetMap + sr.Name + '\', tdLastCopied );
          if not Result then
            Exit;
        end
        else begin
          if FileDateToDateTime( sr.Time ) > tdLastCopied then begin
            CEFileOperations.SourceFilename := sSourceMap + sr.Name;
            CEFileOperations.TargetFilename := sTargetMap + sr.Name;
            CEFileOperations.Operation := coCopyDesktopToCE;
            Result := CEFileOperations.Execute;
            if not Result then begin
              ShowMessage( 'error while copying files to PocketPC' );
              Exit;
            end;
          end;
        end;
      until FindNext(sr) <> 0;
    end;
  finally
    FindClose(sr);
  end;
end;

function TForm1.BatchCopyFiles( sSourceMap, sTargetMap: string; tdLastCopied: TDateTime; bIncludeSubmaps: Boolean = True ): Boolean;
var
  sr: TSearchRec;
  FileAttrs: Integer;
  FileList: TStringList;
begin
  Result := True;
  FileAttrs := 0;
  FileAttrs := FileAttrs + faAnyFile;
  FileList := TStringList.Create;
  try
    FileList.Clear;
    if FindFirst( sSourceMap + '*.*', FileAttrs, sr ) = 0 then begin
      // collect all filenames
      repeat
        if ( ( sr.Attr and faDirectory ) = sr.Attr )
           and ( bIncludeSubmaps )
           and ( sr.Name <> '.' )
           and ( sr.Name <> '..' ) then begin
          Result := Result and BatchCopyFiles( sSourceMap + sr.Name + '\', sTargetMap + sr.Name + '\', tdLastCopied, bIncludeSubmaps );
          if not Result then
            Exit;
        end
        else begin
          if FileDateToDateTime( sr.Time ) > tdLastCopied then begin
            FileList.Append( sSourceMap + sr.Name );
          end;
        end;
      until FindNext(sr) <> 0;
      FindClose(sr);
      // do the actual copy operation
      if FileList.Count > 0 then begin
        CEFileOperations.FileNames.Clear;
        CEFileOperations.FileNames.AddStrings( FileList );
        CEFileOperations.SourceFilename := '';
        CEFileOperations.TargetFilename := '';
        CEFileOperations.TargetDirectory := sTargetMap;
        CEFileOperations.Operation := coMultiDesktopToCE;
        Result := CEFileOperations.Execute;
        if not Result then
          ShowMessage( 'error while copying files to PocketPC' );
      end;
    end;
  finally
    FileList.Free;
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  if BatchCopyFiles( edtDesktopMap.Text, edtPocketMap.Text, StrToDateTime( edtDateTime.Text ), chbIncludeSubmaps.Checked ) then
    // remember last update
    edtDateTime.Text := DateTimeToStr( Now );
  ShowMessage( 'Finished' );
end;

procedure TForm1.edtDateTimeExit(Sender: TObject);
begin
  try
    TLabeledEdit( Sender ).Text := DateTimeToStr( StrToDateTime( TLabeledEdit( Sender ).Text ) );
  except
    ShowMessage( 'Needs a valid date and time !' );
    TLabeledEdit( Sender ).SetFocus;
  end;
end;

procedure TForm1.PathNameExit(Sender: TObject);
begin
  TLabeledEdit( Sender ).Text := IncludeTrailingPathDelimiter( TLabeledEdit( Sender ).Text );
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  SettingsFile.GetSettings( Self );
  if Trim( edtDateTime.Text ) = '' then
    edtDateTime.Text := DateTimeToStr( Now );
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  SettingsFile.PutSettings( Self );
end;

end.
